import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.logging import LoggerABC, LoggingLevelEnum, LoggingSettings
from discord.ext import commands
from gismo_core.abc.bot_service_abc import BotServiceABC
from gismo_core.configuration.bot_settings import BotSettings
from gismo_core.configuration.discord_settings import DiscordSettings
from gismo_core.configuration.server_settings import ServerSettings
from gismo_core.abc.module_service_abc import ModuleServiceABC
from gismo_core.service.module_service import ModuleService


class BotService(BotServiceABC, commands.Bot):

    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        modules: ModuleService,
        discord_settings: DiscordSettings,
        bot_settings: BotSettings,
        env: ApplicationEnvironmentABC,
        logging_st: LoggingSettings,
    ):
        # services
        self._config = config
        self._logger = logger
        self._modules = modules
        self._env = env
        self._logging_st = logging_st
        
        # settings
        self._discord_settings = discord_settings
        self._bot_settings: BotSettings = bot_settings
        
        # setup super
        commands.Bot.__init__(self, command_prefix=bot_settings.prefix, help_command=None, intents=discord.Intents().all())

    async def start_async(self):
        self._logger.trace(__name__, 'Try to connect to discord')

        self.add_cog(self._modules)
        
        await self.start(self._discord_settings.token)
        # continue at on_ready

    async def stop_async(self):
        self._logger.debug(__name__, f'Try to stop {BotService}')
        try:
            pass
            # save data
        except Exception as e:
            self._logger.error(__name__, 'Stop failed', e)
        
    async def on_ready(self):
        self._logger.info(__name__, 'Connected to discord')
        
        self._logger.debug(__name__, 'Try to load discord server configs')
        for server in self._bot_settings.servers:
            server: ServerSettings = server
            self._logger.trace(__name__, f'Try to load config for server: {server.id}')
            self._config.add_configuration(f'DSERVER_{server.id}', server)
            self._logger.trace(__name__, f'Loaded config for server: {server.id}')

        self._logger.header(f'{self.user.name}:')
        if self._logging_st.console.value >= LoggingLevelEnum.INFO.value:
            Console.banner(self._env.application_name if self._env.application_name != '' else 'Gismo')
        
        await self._modules.on_ready()
